package com.hungraim.ism.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hungraim.ism.dao.*;
import com.hungraim.ism.pojo.vo.disable.DisableExamineInfoVO;
import com.hungraim.ism.pojo.vo.disable.DisableVO;
import com.hungraim.ism.pojo.vo.disable.ExamineRecord;
import com.hungraim.ism.service.DisableService;
import com.hungraim.ltc.pojo.entity.disable.*;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

@Service
public class DisableServiceImpl implements DisableService {

    @Autowired
    private DisableApplyMapper disableApplyMapper;
    @Autowired
    private FndDistrictMapper districtMapper;
    @Autowired
    private FndOrganMapper fndOrganMapper;
    @Autowired
    private FndStreetMapper fndStreetMapper;
    @Autowired
    private FndSrvModeMapper fndSrvModeMapper;
    @Autowired
    private SrvOrganMapper srvOrganMapper;
    @Autowired
    private FndDiseaseTypeMapper fndDiseaseTypeMapper;
    @Autowired
    private DisableApplyHisMapper disableApplyHisMapper;

    /**
     * 获取机构ids
     *
     * @param organId  父机构id
     * @param organIds 机构list
     */
    private void getOrganList(String organId, List<String> organIds) {
        QueryWrapper<ChFndOrgan> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("STATUS", "1");
        queryWrapper.eq("PARENT_ID", organId);
        List<ChFndOrgan> organs = fndOrganMapper.selectList(queryWrapper);
        if (organs != null && organs.size() != 0) {
            for (ChFndOrgan chFndOrgan : organs) {
                String childOrganId = chFndOrgan.getOrganId();
                organIds.add(childOrganId);
                if ("1".equals(chFndOrgan.getIsLeaf())) {
                    getOrganList(childOrganId, organIds);
                }
            }
        }
    }


    @Override
    public Page<DisableVO> disableList(String organId, String districtProvincial, String districtCity,
                                       String districtArea, String realName, String certiCode, String dataFrom,
                                       String srvOrganName, int page, int pageSize) {
        // 没有organId参数的时候，这个集合就为null
        List<String> organIds = null;
        if (StrUtil.isNotBlank(organId)) {
            organIds = Arrays.asList(organId.split(","));
        }
        return disableApplyMapper.disableList(new Page<>(page, pageSize), organIds, districtProvincial,
                districtCity, districtArea, realName, certiCode, dataFrom, srvOrganName);
    }

    @Override
    public DisableExamineInfoVO disableExamineInfo(String applyId) {
        DisableExamineInfoVO disableExamineInfoVO = disableApplyMapper.selectDisableExamineInfoByApplyId(applyId);
        List<ExamineRecord> examineRecords = disableApplyMapper.listExamineRecordByApplyId(applyId);
        disableExamineInfoVO.setExamineRecords(examineRecords);
        return disableExamineInfoVO;
    }

    @Override
    public void addDisable(ChDisableApply chDisableApply) {
        disableApplyMapper.insert(chDisableApply);
    }

    @Override
    public void disableExamine(String applyId, String checkName, Short applyStatus, String checkRemark) {
        ChDisableApply chDisableApply = disableApplyMapper.selectById(applyId);
        Date date = new Date();
        ChDisableApplyHis chDisableApplyHis = new ChDisableApplyHis();
        BeanUtils.copyProperties(chDisableApply, chDisableApplyHis);
        chDisableApplyHis.setCheckName(checkName);
        chDisableApplyHis.setApplyStatus(applyStatus);
        chDisableApplyHis.setCheckRemark(checkRemark);
        chDisableApplyHis.setCheckTime(date);
        if (applyStatus.equals(Short.valueOf("2"))) {
            // 审核通过
            chDisableApply.setCheckName(checkName);
            chDisableApply.setApplyStatus(applyStatus);
            chDisableApply.setCheckRemark(checkRemark);
            chDisableApply.setCheckTime(date);
            disableApplyMapper.updateById(chDisableApply);
        }
        disableApplyHisMapper.insert(chDisableApplyHis);
    }

    @Override
    public List<ChFndDistrict> district(String districtLevel, String districtCode) {
        if ("4".equals(districtLevel)) {
            //查询街道
            QueryWrapper<ChFndStreet> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("IS_VALID", 0);
            queryWrapper.eq("AREA_CODE", districtCode);
            List<ChFndStreet> fndStreets = fndStreetMapper.selectList(queryWrapper);
            List<ChFndDistrict> resuts = new ArrayList<>();
            for (ChFndStreet chFndStreet : fndStreets) {
                ChFndDistrict chFndDistrict = new ChFndDistrict();
                chFndDistrict.setDistrictCode(chFndStreet.getStreetCode());
                chFndDistrict.setDistrictName(chFndStreet.getStreetName());
                chFndDistrict.setParentDistirctCode(chFndStreet.getAreaCode());
                resuts.add(chFndDistrict);
            }
            return resuts;
        } else {
            QueryWrapper<ChFndDistrict> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("IS_VALID", "Y");
            queryWrapper.eq("DISTRICT_LEVEL", districtLevel);
            if ("1".equals(districtLevel)) {
                //查出所有的省
                return districtMapper.selectList(queryWrapper);
            } else {
                //查询市/县
                queryWrapper.eq("PARENT_DISTIRCT_CODE", districtCode);
                return districtMapper.selectList(queryWrapper);
            }
        }
    }

    @Override
    public List<ChFndOrgan> fndOrganRoot() {
        QueryWrapper<ChFndOrgan> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("IS_LEAF", "1");
        queryWrapper.and(wrapper -> wrapper.isNull("PARENT_ID").or().eq("PARENT_ID", ""));
        return fndOrganMapper.selectList(queryWrapper);
    }

    @Override
    public List<ChFndSrvMode> fndSrvModeList() {
        return fndSrvModeMapper.selectList(new QueryWrapper<>());
    }

    @Override
    public List<ChSrvOrgan> srvOrganList(String organId) {
        return srvOrganMapper.selectList(new QueryWrapper<ChSrvOrgan>().eq("organ_id", organId));
    }

    @Override
    public List<ChFndDiseaseType> fndDiseaseList(String parDiseTypeId) {
        QueryWrapper<ChFndDiseaseType> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("IS_VALID", "0");
        if (StringUtils.isNotBlank(parDiseTypeId)) {
            queryWrapper.eq("PAR_DISE_TYPE_ID", parDiseTypeId);
        } else {
            queryWrapper.and(wrapper -> wrapper.isNull("PAR_DISE_TYPE_ID").or().eq("PAR_DISE_TYPE_ID", ""));
        }
        return fndDiseaseTypeMapper.selectList(queryWrapper);
    }

    @Override
    public String subsidiariesOrganId(String organId) {
        List<String> organIds = new ArrayList<>();
        organIds.add(organId);
        // 获取子孙机构集合
        this.getOrganList(organId, organIds);
        StringBuilder sb = new StringBuilder();
        for (String o : organIds) {
            sb.append(o).append(",");
        }
        sb.deleteCharAt(sb.length() - 1);
        return sb.toString();
    }

    @Override
    public void updateDisable(ChDisableApply chDisableApply) {
        disableApplyMapper.updateById(chDisableApply);
    }
}